
file1 = open("rhum.z1.1970_2000.STD.txt")
file2 = open("rhum.z2.1970_2000.STD.txt")
file3 = open("shum.z1.1970_2000.STD.txt")
file4 = open("shum.z2.1970_2000.STD.txt")
file5 = open("slp.1970_2000.STD.txt")
file6 = open("temp.1970_2000.STD.txt")
file7 = open("uwnd.z1.1970_2000.STD.txt")
file8 = open("uwnd.z2.1970_2000.STD.txt")
file9 = open("vwnd.z1.1970_2000.STD.txt")
file10 = open("vwnd.z2.1970_2000.STD.txt")
file11 = open("z500.1970_2000.STD.txt")


lines1 = file1.readlines()
lines2 = file2.readlines()
lines3 = file3.readlines()
lines4 = file4.readlines()
lines5 = file5.readlines()
lines6 = file6.readlines()
lines7 = file7.readlines()
lines8 = file8.readlines()
lines9 = file9.readlines()
lines10 = file10.readlines()
lines11 = file11.readlines()

i = 0
for line in lines1:
  print line[:-1],lines2[i][:-1],lines3[i][:-1],lines4[i][:-1],lines5[i][:-1],lines6[i][:-1],lines7[i][:-1],lines8[i][:-1],lines9[i][:-1],lines10[i][:-1],lines11[i][:-1]
  i = i + 1

  
